@@ -1,9 +1,9 @@
 #' Draw samples from the posterior distribution of an estimated model
 #'
 #' @param model a fitted model of the supported types
-#' @param newdata data frame; new observations at which the posterior draws
+#' @param data  data frame; new observations at which the posterior draws
 #'   from the model should be evaluated. If not supplied, the data used to fit
-#'   the model will be used for `newdata`, if available in `model`.
+#'   the model will be used for `data`, if available in `model`.
 #' @param n numeric; the number of posterior samples to return.
 #' @param seed numeric; a random seed for the simulations.
 #' @param scale character;
@@ -13,7 +13,7 @@
 #' @param unconditional logical; if `TRUE` (and `freq == FALSE`) then the
 #'   Bayesian smoothing parameter uncertainty corrected covariance matrix is
 #'   used, if available.
-#' @param weights numeric; a vector of prior weights. If `newdata` is null
+#' @param weights numeric; a vector of prior weights. If `data` is null
 #'   then defaults to `object[["prior.weights"]]`, otherwise a vector of ones.
 #' @param ncores number of cores for generating random variables from a
 #'   multivariate normal distribution. Passed to [mvnfast::rmvn()].
@@ -21,14 +21,15 @@
 #'   to work on Windows with current `R`).
 #' @param ... arguments passed to other methods. For `fitted_samples()`, these
 #'   are passed on to `predict.gam()`.
+#' @param newdata Deprecated: use `data` instead.
 #'
 #' @return A tibble (data frame) with 3 columns containing the posterior
 #'   predicted values in long format. The columns are
-#' * `row` (integer) the row of `newdata` that each posterior draw relates to,
+#' * `row` (integer) the row of `data` that each posterior draw relates to,
 #' * `draw` (integer) an index, in range `1:n`, indicating which draw each row
 #'     relates to,
 #' * `response` (numeric) the predicted response for the indicated row of
-#'     `newdata`.
+#'     `data`.
 #'
 #' @author Gavin L. Simpson
 #'
@@ -45,10 +46,11 @@
 
 #' @export
 #' @rdname posterior_samples
-`posterior_samples.gam` <- function(model, n, newdata, seed,
+`posterior_samples.gam` <- function(model, n, data = newdata, seed,
                                     scale = c("response","linear_predictor"),
                                     freq = FALSE, unconditional = FALSE,
-                                    weights = NULL, ncores = 1L, ...) {
+                                    weights = NULL, ncores = 1L, ...,
+                                    newdata = NULL) {
     .NotYetImplemented()
 }
 
@@ -67,11 +69,11 @@
 #'
 #' @return A tibble (data frame) with 3 columns containing the posterior
 #'   predicted values in long format. The columns are
-#' * `row` (integer) the row of `newdata` that each posterior draw relates to,
+#' * `row` (integer) the row of `data` that each posterior draw relates to,
 #' * `draw` (integer) an index, in range `1:n`, indicating which draw each row
 #'     relates to,
 #' * `response` (numeric) the predicted response for the indicated row of
-#'     `newdata`.
+#'     `data`.
 #'
 #' @author Gavin L. Simpson
 #'
@@ -113,11 +115,11 @@
 #' fs
 #' }
 #' \dontshow{options(op)}
-`fitted_samples.gam` <- function(model, n = 1, newdata, seed,
+`fitted_samples.gam` <- function(model, n = 1, data = newdata, seed,
                                  scale = c("response", "linear_predictor"),
                                  method = c("gaussian", "mh", "inla"),
                                  freq = FALSE, unconditional = FALSE,
-                                 ncores = 1L, ...) {
+                                 ncores = 1L, ..., newdata = NULL) {
     if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
         runif(1)
     }
@@ -130,8 +132,12 @@
         on.exit(assign(".Random.seed", R.seed, envir = .GlobalEnv))
     }
 
-    if (missing(newdata) || is.null(newdata)) {
-        newdata <- model[["model"]]
+    if (!is.null(newdata)) {
+        newdata_deprecated()
+    }
+
+    if (is.null(data)) {
+        data <- model[["model"]]
     }
 
     scale <- match.arg(scale)
@@ -144,7 +150,7 @@
     V <- get_vcov(model, frequentist = freq, unconditional = unconditional)
     Rbeta <- rmvn(n = n, mu = coef(model), sigma = V, ncores = ncores)
     ## don't need to pass freq, unconditional here as that is done for V
-    Xp <- predict(model, newdata = newdata, type = "lpmatrix", ...)
+    Xp <- predict(model, newdata = data, type = "lpmatrix", ...)
     sims <- Xp %*% t(Rbeta)
 
     if (isTRUE(identical(scale, "response"))) {
@@ -174,11 +180,11 @@
 #'
 #' @return A tibble (data frame) with 3 columns containing the posterior
 #'   predicted values in long format. The columns are
-#' * `row` (integer) the row of `newdata` that each posterior draw relates to,
+#' * `row` (integer) the row of `data` that each posterior draw relates to,
 #' * `draw` (integer) an index, in range `1:n`, indicating which draw each row
 #'     relates to,
 #' * `response` (numeric) the predicted response for the indicated row of
-#'     `newdata`.
+#'     `data`.
 #'
 #' @author Gavin L. Simpson
 #'
@@ -206,9 +212,11 @@
 #' ## Exclude s(x1)
 #' predicted_samples(m, n = 5, newd, exclude = "s(x1)", seed = 25)
 #'
-#' ## Select which terms --- result same as previous
+#' ## Select which terms --- result should be the same as previous
+#' ## but note that we have to include any parametric terms, including the
+#' ## constant term
 #' predicted_samples(m, n = 5, newd, seed = 25,
-#'                   terms = c("s(x0)", "s(x2)", "s(x3)"))
+#'                   terms = c("Intercept", "s(x0)", "s(x2)", "s(x3)"))
 #' \dontshow{options(op)}
 `predicted_samples` <- function(model, ...) {
     UseMethod("predicted_samples")
@@ -224,9 +232,13 @@
 #' @rdname predicted_samples
 #' @importFrom tibble as_data_frame add_column
 #' @importFrom tidyr gather
-`predicted_samples.gam` <- function(model, n = 1, newdata = NULL, seed = NULL,
-                                    weights = NULL, ...) {
-    sims <- simulate(model, nsim = n, seed = seed, newdata = newdata,
+`predicted_samples.gam` <- function(model, n = 1, data = newdata, seed = NULL,
+                                    weights = NULL, ..., newdata = NULL) {
+    if (!is.null(newdata)) {
+        newdata_deprecated()
+    }
+
+    sims <- simulate(model, nsim = n, seed = seed, data = data,
                      weights = weights, ...)
     RNGstate <- attr(sims, "seed")
     colnames(sims) <- paste0(".V", seq_len(NCOL(sims)))
@@ -263,20 +275,14 @@
 #' * `by_variable`; character vector. If the smooth involves a `by` term, the
 #'     by variable will be named here, `NA_character_` otherwise.
 #' * `row`; integer. A vector of values `seq_len(n_vals)`, repeated if
-#'     `n > 1L`. Indexes the row in `newdata` for that particular draw.
+#'     `n > 1L`. Indexes the row in `data` for that particular draw.
 #' * `draw`; integer. A vector of integer values indexing the particular
 #'     posterior draw that each row belongs to.
 #' * `value`; numeric. The value of smooth function for this posterior draw
 #'     and covariate combination.
-#' * `.xN`; numeric. A series of one or more columns containing data required
-#'     for the smooth. `.x1` will always be present and contains the values of
-#'     the covariate in the smooth. For example if `smooth` is `s(z)` then
-#'     `.x1` will contain the values of covariate `z` at which the smooth was
-#'     evaluated. Further covariates for multi-dimensional thin plate splines
-#'     (e.g. `s(x, z)`) or tensor product smooths (e.g. `te(x,z,a)`) will
-#'     result in variables `.x1` and `.x2`, and `.x1`, `.x2`, and `.x3`
-#'     respectively, with the number (`1`, `2`, etc) representing the order
-#'     in which the covariates were specified in the smooth.
+#' * `xxx`; numeric. A series of one or more columns containing data required
+#'     for the smooth, named as per the variables involved in the respective
+#'     smooth.
 #' * Additional columns will be present in the case of factor by smooths,
 #'     which will contain the level for the factor named in `by_variable` for
 #'     that particular posterior draw.
@@ -314,7 +320,7 @@
 }
 
 #' @param n_vals numeric; how many locations to evaluate the smooth at if
-#'   `newdata` not supplied
+#'   `data` not supplied
 #' @param term character; select which smooth's posterior to draw from.
 #'   The default (`NULL`) means the posteriors of all smooths in `model`
 #'   wil be sampled from. If supplied, a character vector of requested terms.
@@ -332,12 +338,13 @@
 #' @importFrom mvnfast rmvn
 #' @importFrom dplyr bind_rows starts_with
 #' @importFrom tibble as_tibble add_column
-#' @importFrom tidyr gather
+#' @importFrom tidyr pivot_longer
 #' @importFrom mgcv PredictMat
-`smooth_samples.gam` <- function(model, term = NULL, n = 1, newdata = NULL,
+`smooth_samples.gam` <- function(model, term = NULL, n = 1, data = newdata,
                                  seed = NULL, freq = FALSE,
                                  unconditional = FALSE,
-                                 ncores = 1L, n_vals = 200, ...) {
+                                 ncores = 1L, n_vals = 200, ...,
+                                 newdata = NULL) {
     if (!exists(".Random.seed", envir = .GlobalEnv, inherits = FALSE)) {
         runif(1)
     }
@@ -373,9 +380,13 @@
         stop("No smooths left that can be sampled from.")
     }
 
-    need_newdata <- FALSE
-    if (is.null(newdata)) {
-        need_newdata <- TRUE
+    if (!is.null(newdata)) {
+        newdata_deprecated()
+    }
+
+    need_data <- FALSE
+    if (is.null(data)) {
+        need_data <- TRUE
     }
 
     V <- get_vcov(model, frequentist = freq, unconditional = unconditional)
@@ -384,16 +395,16 @@
 
     sims <- data_names <- vector('list', length = length(S))
     for (i in seq_along(S)) {
-        if (need_newdata) {
+        if (need_data) {
             # FIXME: should offset be NULL?
-            newdata <- smooth_data(model, id = take[i], n = n_vals,
-                                   offset = NULL)
+            data <- smooth_data(model, id = take[i], n = n_vals,
+                                offset = NULL)
             ## I don't think we need offset here as that really just shifts the
             ## response around
         }
         sm  <- get_smooths_by_id(model, take[i])[[1L]]
-        idx <- smooth_coefs(sm)
-        Xp <- PredictMat(sm, data = newdata)
+        idx <- smooth_coef_indices(sm)
+        Xp <- PredictMat(sm, data = data)
         betas <- rmvn(n = n, mu = coefs[idx], sigma = V[idx, idx, drop=FALSE],
                       ncores = ncores)
         simu <- Xp %*% t(betas)
@@ -404,32 +415,37 @@
         if (is_fac_by) {
             simu <- add_factor_by_data(simu, n = n_vals,
                                        by_name = by_variable(sm),
-                                       by_data = newdata, before = 1L)
+                                       by_data = data, before = 1L)
         } else {
             simu <- add_column(simu,
                                by_variable = rep(NA_character_,
                                                  times = nr_simu))
         }
-        simu <- add_smooth_var_data(simu, smooth_variable(sm), newdata)
+    #   # add on spline type info
+        sm_type <- smooth_type(sm)
+        simu <- add_column(simu,
+            smooth = rep(unlist(lapply(strsplit(S[[i]], ":"), `[`, 1L)),
+            nr_simu),
+            term = rep(S[[i]], each = nr_simu),
+            type = rep(sm_type, nr_simu),
+            row = seq_len(nr_simu),
+            .after = 0L)
+        simu <- bind_cols(simu, data[smooth_variable(sm)])
+        simu <- pivot_longer(simu, cols = dplyr::starts_with("..V"),
+            names_to = "draw", values_to = "value",
+            names_transform = \(x) as.integer(sub("\\.\\.V", "", x)))
+        # summ_names <- names(data[!vapply(data, is.factor, logical(1))])
+        # names(summ_names) <- paste0(".x", seq_along(summ_names))
+        # data_names[[i]] <- summ_names
+        ## nest all columns with varying data
+        simu <- nest(simu, data = all_of(c("row", "draw", "value",
+            smooth_variable(sm))))
         sims[[i]] <- simu
-        summ_names <- names(newdata[!vapply(newdata, is.factor, logical(1))])
-        names(summ_names) <- paste0(".x", seq_along(summ_names))
-        data_names[[i]] <- summ_names
     }
 
-    sims <- do.call("bind_rows", sims)
-    sims <- add_column(sims,
-                       smooth = rep(unlist(lapply(strsplit(S, ":"), `[`, 1L)),
-                                    each = nr_simu),
-                       .before = 1L)
-    sims <- add_column(sims, term = rep(S, each = nr_simu), .before = 2L)
-    sims <- add_column(sims, row = rep(seq_len(nrow(newdata)),
-                       times = length(S)))
-    sims <- gather(sims, key = "draw", value = "value",
-                   dplyr::starts_with("..V"))
-    sims[["draw"]] <- as.integer(sub("\\.\\.V", "", sims[["draw"]]))
+    sims <- bind_rows(sims)
+    sims <- unnest(sims, all_of("data"))
     attr(sims, "seed") <- RNGstate
-    attr(sims, "data_names") <- setNames(data_names, nm = S)
     ## add classes
     class(sims) <- c("smooth_samples", "posterior_samples", class(sims))
     sims
