@@ -3,8 +3,6 @@
 ## load packages
 library("testthat")
 library("gratia")
-library("mgcv")
-library('gamm4')
 
 ## first derivatives of all smooths...
 test_that("Point-wise confidence interval for a first derivatives of a GAM works", {
@@ -35,7 +33,7 @@ test_that("Point-wise confidence interval for a GAM works", {
     ci <- confint(m_gam, parm = "s(x1)", type = "confidence")
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -45,7 +43,7 @@ test_that("Simultaneous interval for a GAM works", {
     ci <- confint(m_gam, parm = "s(x1)", type = "simultaneous", nsim = 100)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -55,7 +53,7 @@ test_that("Point-wise confidence interval for a 2d smooth works", {
     ci <- confint(su_m_bivar_te, parm = "te(x,z)", type = "confidence")
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x", "z", "est",
+    expect_named(ci, expected = c("smooth", "type", "by", "x", "z", "est",
                                   "se", "crit", "lower", "upper"))
 })
 
@@ -66,7 +64,7 @@ test_that("Simultaneous interval for a 2d smooth works", {
                   nsim = 100)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x", "z", "est",
+    expect_named(ci, expected = c("smooth", "type", "by", "x", "z", "est",
                                   "se", "crit", "lower", "upper"))
 })
 
@@ -75,7 +73,7 @@ test_that("Point-wise confidence interval for a GAMM works", {
     ci <- confint(m_gamm, parm = "s(x1)", type = "confidence")
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -85,7 +83,7 @@ test_that("Simultaneous interval for a GAMM works", {
     ci <- confint(m_gamm, parm = "s(x1)", type = "simultaneous", nsim = 100)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -96,8 +94,8 @@ test_that("Point-wise confidence interval for a GAM with factor by variable work
                   partial_match = TRUE)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x2", "est", "se",
-                                  "fac", "crit", "lower", "upper"))
+    expect_named(ci, expected = c("smooth", "type", "by", "x2", "fac", "est",
+                                  "se", "crit", "lower", "upper"))
     expect_equal(paste0("s(x2):fac", levels(su_eg4[["fac"]])),
                  unique(ci[["smooth"]]))
 })
@@ -108,8 +106,8 @@ test_that("Simultaneous confidence interval for a GAM with factor by variable wo
                   partial_match = TRUE)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x2", "est", "se",
-                                  "fac", "crit", "lower", "upper"))
+    expect_named(ci, expected = c("smooth", "type", "by", "x2", "fac", "est",
+                                  "se", "crit", "lower", "upper"))
     expect_equal(paste0("s(x2):fac", levels(su_eg4[["fac"]])),
                  unique(ci[["smooth"]]))
 })
@@ -120,8 +118,8 @@ test_that("Point-wise confidence interval for a GAM with selected factor by vari
     ci <- confint(su_m_factor_by_x2, parm = "s(x2):fac1", type = "confidence")
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x2", "est", "se",
-                                  "fac", "crit", "lower", "upper"))
+    expect_named(ci, expected = c("smooth", "type", "by", "x2", "fac", "est",
+                                  "se", "crit", "lower", "upper"))
 })
 
 test_that("Point-wise confidence interval for a GAMM works", {
@@ -129,7 +127,7 @@ test_that("Point-wise confidence interval for a GAMM works", {
     ci <- confint(m_gamm4, parm = "s(x1)", type = "confidence")
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -139,7 +137,7 @@ test_that("Simultaneous interval for a GAMM works", {
     ci <- confint(m_gamm4, parm = "s(x1)", type = "simultaneous", nsim = 100)
     expect_s3_class(ci, "confint.gam")
     expect_s3_class(ci, "tbl_df")
-    expect_named(ci, expected = c("smooth", "by_variable", "x1", "est", "se",
+    expect_named(ci, expected = c("smooth", "type", "by", "x1", "est", "se",
                                   "crit", "lower", "upper"))
 })
 
@@ -164,7 +162,7 @@ test_that("confint.fderiv example output", {
     ci <- confint(fd, type = "confidence")
     expect_snapshot_output(ci)
     ## simultaneous interval for smooth term of x2
-    set.seed(42)
+    set.seed(24)
     x2_sint <- confint(fd, parm = "x2", type = "simultaneous",
                        nsim = 10000, ncores = 2)
     expect_snapshot_output(x2_sint)
