@@ -17,10 +17,12 @@ library("datasets")
 data(CO2, package = "datasets")
 CO2 <- transform(CO2, Plant_uo = factor(Plant, ordered = FALSE))
 data(bird_move, package = "gratia")
-ctrl <- gam.control(nthreads = 3)
 data(zooplankton, package = "gratia")
 zooplankton <- transform(zooplankton, year_f = factor(year))
 
+## use several threads to speed up some fits
+ctrl <- gam.control(nthreads = 3)
+
 ## the first training and testing data set will be used to compare dynamics of
 ## plankton communities in Lake Mendota
 zoo_train <- subset(zooplankton, year%%2==0 & lake=="Mendota")
@@ -43,6 +45,8 @@ test_that("draw() can plot CO2 model 1", {
                     control = ctrl)
     plt <- draw(CO2_mod1, overall_uncertainty = TRUE)
     expect_doppelganger("hgam-paper-co2-model-1", plt)
+
+    expect_silent(d <- derivatives(CO2_mod1))
 })
 
 test_that("draw() can plot CO2 model 2", {
@@ -58,6 +62,8 @@ test_that("draw() can plot CO2 model 2", {
     )
     plt <- draw(CO2_mod2, overall_uncertainty = TRUE)
     expect_doppelganger("hgam-paper-co2-model-2", plt)
+
+    expect_silent(d <- derivatives(CO2_mod2))
 })
 
 ## We show smooths 1, 14, 3, 5, 10, 13 in the paper code
@@ -72,6 +78,8 @@ test_that("draw() can plot CO2 model 3", {
                     control = ctrl)
     plt <- draw(CO2_mod3, overall_uncertainty = TRUE)
     expect_doppelganger("hgam-paper-co2-model-3", plt)
+
+    expect_silent(d <- derivatives(CO2_mod3))
 })
 
 test_that("draw() can plot CO2 model 4", {
@@ -84,6 +92,8 @@ test_that("draw() can plot CO2 model 4", {
                     control = ctrl)
     plt <- draw(CO2_mod4, overall_uncertainty = TRUE)
     expect_doppelganger("hgam-paper-co2-model-4", plt)
+
+    expect_silent(d <- derivatives(CO2_mod4))
 })
 
 test_that("draw() can plot CO2 model 5", {
@@ -97,6 +107,8 @@ test_that("draw() can plot CO2 model 5", {
                     control = ctrl)
     plt <- draw(CO2_mod5, overall_uncertainty = TRUE)
     expect_doppelganger("hgam-paper-co2-model-5", plt)
+
+    expect_silent(d <- derivatives(CO2_mod5))
 })
 
 ## bird_move
@@ -148,16 +160,15 @@ test_that("draw() throws message with bird_move model 4", {
     skip_on_cran()
     skip_on_travis()
     skip_on_ci()
-    bird_mod4 <- gam(count ~ t2(week, latitude, species, 
+    bird_mod4 <- gam(count ~ t2(week, latitude, species,
                                 bs = c("cc", "tp", "re"),
                                 k = c(10, 10, 6), m = c(2, 2, 2)),
                      data = bird_move, method = "REML", family = poisson(),
                      knots = list(week = c(0, 52)),
                      control = ctrl)
     ## There's nothing we can currently do, as
-    expect_message(draw(bird_mod4),
-                   "Unable to draw any of the model terms.",
-                   fixed = FALSE)
+    expect_silent(plt <- draw(bird_mod4, n = 25, rug = FALSE))
+    expect_doppelganger("hgam-paper-bird-move-model-4", plt)
 })
 
 test_that("draw() can plot bird_move model 5", {
